 /**
 ********************************************************************************
 * Copyright (c) 2017 - 2018 Robert Bosch GmbH.
 * 
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *     Robert Bosch GmbH - initial API and implementation
 ********************************************************************************
 */
package org.eclipse.app4mc.emfutils.content.extractor.builders;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Pattern;

import org.eclipse.emf.common.util.BasicEMap;
import org.eclipse.emf.common.util.EMap;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.EStructuralFeature.Setting;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.util.EcoreUtil;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;

public class EObjectRefsBufferBuilder {

	private EMap<String, Object> id2ObjectsMap;

	public StringBuffer buildBuffer(final EObject eObject) {

		this.id2ObjectsMap = new BasicEMap<String, Object>();

		final StringBuffer plantumlBuffer = new StringBuffer();


		plantumlBuffer.append("@startuml");
		plantumlBuffer.append(System.getProperty("line.separator"));

		buildBuffer(eObject, plantumlBuffer);

		plantumlBuffer.append(System.getProperty("line.separator"));
		plantumlBuffer.append("@enduml");

		return plantumlBuffer;

	}


	public EMap<String, Object> getId2ObjectsMap() {
		return this.id2ObjectsMap;
	}


	private void buildBuffer(final EObject selectedEObj, final StringBuffer plantumlBuffer) {

		final List<EObject> result = new ArrayList<EObject>();

		/*- getting all the elements from the selected objects EClass --> by supplying the EResource of the corresponding EClass (i.e. ecore or Xcore) */

		Collection<Setting> allRefSettings = null;

		final Resource eResource = selectedEObj.eResource();

		if (eResource == null) {
			allRefSettings = EcoreUtil.UsageCrossReferencer.find(selectedEObj,
					EcoreUtil.getRootContainer(selectedEObj));
		}
		else {
			allRefSettings = EcoreUtil.UsageCrossReferencer.find(selectedEObj, eResource.getResourceSet());
		}

		/*-
		 * In case of many elements it is difficult to visualize the contents in the diagram.
		 *
		 * Below code provides the possibility to specify the range
		 */
		if (allRefSettings.size() > 100) {


			final InputDialog inputDialog = new InputDialog(
					org.eclipse.swt.widgets.Display.getDefault().getActiveShell(), "Element selection dialog",
					"There are " + allRefSettings.size()
							+ " references. specify the range for which contents should be represented graphically",
					"0-" + (allRefSettings.size() - 1), new IInputValidator() {

						@Override
						public String isValid(final String newText) {
							if (Pattern.matches("(\\d)+\\-(\\d)+", newText) == false) {
								return "Specify the range in the following way int-int eg. 10-20 ";
							}
							return null;
						}
					});

			final int status = inputDialog.open();

			if (status == 0) {
				final String value = inputDialog.getValue();

				final String[] split = value.split("-");

				allRefSettings = new ArrayList<Setting>(allRefSettings).subList(Integer.parseInt(split[0]),
						Integer.parseInt(split[1]) < allRefSettings.size() ? Integer.parseInt(split[1])
								: (allRefSettings.size() - 1));

			}
			else {
				allRefSettings = new ArrayList<Setting>(allRefSettings).subList(0, 99);
			}


		}


		/*- if refs of the EClass are found, then fill the plantUMLBuffer with the appropriate contents */

		if (allRefSettings.size() > 0) {

			for (final EStructuralFeature.Setting setting : allRefSettings) {

				if (!isElementContained(result, setting.getEObject())) {
					result.add(setting.getEObject());

				}

			}

			createClass(plantumlBuffer, this.id2ObjectsMap, selectedEObj);

			for (final EObject eobj : result) {

				createClass(plantumlBuffer, this.id2ObjectsMap, eobj);

				if (!isNamePresent(eobj)) {

					/*- As name is not present, generate the heirarchy till the level Object with Structural Feature name is present */

					associateContainer(eobj, plantumlBuffer, this.id2ObjectsMap);


					plantumlBuffer.append(System.getProperty("line.separator"));
				}
			}

			for (final EObject eobj : result) {

				final String name = getName(eobj).toString();

				plantumlBuffer.append(name + "--> " + getName(selectedEObj));

				plantumlBuffer.append(System.getProperty("line.separator"));
			}
		}
		else {

			/*- no references for this EClass are found */

			createClass(plantumlBuffer, this.id2ObjectsMap, selectedEObj);

		}


	}

	private void createClass(final StringBuffer plantumlBuffer, final EMap<String, Object> id2ObjectsMap,
			final EObject selectedEObj) {
		final String uuid = EcoreUtil.generateUUID();

		plantumlBuffer.append("class " + getName(selectedEObj) + " << (O,#B4A7E5) " + selectedEObj.eClass().getName()
				+ " >>" + "    [[" + uuid + "]]");

		id2ObjectsMap.put(uuid, selectedEObj);

		plantumlBuffer.append(System.getProperty("line.separator"));
	}


	private Object getName(final EObject eObject) {
		final EStructuralFeature eStructuralFeature = eObject.eClass().getEStructuralFeature("name");

		if (eStructuralFeature != null) {

			final Object originalName = eObject.eGet(eStructuralFeature);
			
			if(originalName == null || originalName.toString().length()==0) {
				return eObject.eClass().getName() + "__" + eObject.hashCode();
			}

			return "\"" + originalName + "\"";
		}

		return eObject.eClass().getName() + "__" + eObject.hashCode();
	}


	private void associateContainer(final EObject eobj, final StringBuffer plantumlBuffer,
			final EMap<String, Object> id2ObjectsMap) {

		final EObject eContainerObj = eobj.eContainer();

		if (eContainerObj == null) {
			return;
		}

		createClass(plantumlBuffer, id2ObjectsMap, eContainerObj);


		plantumlBuffer.append(getName(eContainerObj) + " *-- " + getName(eobj));

		plantumlBuffer.append(System.getProperty("line.separator"));

		if (!isNamePresent(eContainerObj)) {

			associateContainer(eContainerObj, plantumlBuffer, id2ObjectsMap);

		}

	}

	private boolean isNamePresent(final EObject eObject) {
		final EStructuralFeature eStructuralFeature = eObject.eClass().getEStructuralFeature("name");

		if (eStructuralFeature == null) {

			return false;
		}

		return true;
	}

	private boolean isElementContained(final List<EObject> l, final EObject eRef) {
		boolean isElementFound = false;
		final Iterator<EObject> it = l.iterator();
		while (it.hasNext() && !isElementFound) {
			final EObject isContained = it.next();
			if (isContained.equals(eRef)) {
				isElementFound = true;
			}
		}
		return isElementFound;
	}
}
